#!/bin/bash
#
# Copyright (C) 2015 DECOIT GmbH
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

####
#Dieses Icinga-Plugin liest die von einer Snort-Instanz generierten
#Alarmmeldungen aus der Datenbank aus und übermittelt diese mit Hilfe
#des send_nsca an Icinga.
###

# Zugangsdaten für die Snort-DB.
SNORT_DB="snort"
SNORT_DB_HOST="localhost"
SNORT_DB_PORT="3306"
SNORT_DB_USER="snort"
SNORT_DB_PASS="snort"

#Parameter für send_nsca festlegen.
ICINGA_HOST="10.240.1.20"
HOSTNAME="iMonitor-Sensors"
SERVICE_DESCRIPTION="Snort Alerts"
SERVICE_STATE="0"


# Sucht in der Snort-DB nach der aktuellen Event-ID und startet im Anschluss die Event-ID-Abfragen.
function get_last_snort_id(){

	# SQL-Statements zum Auslesen der Event-IDs.
	mysql_exec_event_id="select max(cid) from event"
	mysql_exec_event_ids="SELECT cid FROM event ORDER BY cid DESC LIMIT 10"

	#Auslesen der aktuellen Event-ID aus der Snort-Datenbank
	EVENT_ID=`mysql --skip-column-names -h $SNORT_DB_HOST -P $SNORT_DB_PORT -u $SNORT_DB_USER --password=$SNORT_DB_PASS -e "$mysql_exec_event_id" $SNORT_DB`

	while true; do
		get_ids
	done

}

# Liest die letzten 10 Event-IDs aus der Snort-DB und überprüft, welche davon verarbeitet wurden.
function get_ids(){

	# Abfrage der Event-IDs
	EVENT_ID_NEW=`mysql --skip-column-names -h $SNORT_DB_HOST -P $SNORT_DB_PORT -u $SNORT_DB_USER --password=$SNORT_DB_PASS -e "$mysql_exec_event_id" $SNORT_DB`

		# Abgleich mit der letzten verarbeiteten Event-ID. Bei neuer Event-ID wird diese und alle weiteren Events verarbeitet.
		if (($EVENT_ID_NEW > $EVENT_ID))
			then
			build_sql_statements
			EVENT_ID=$EVENT_ID_NEW
		fi
}

# Vorbereitung der SQL-Statements mit der aktuellen Event-ID
function build_sql_statements(){

	mysql_exec_timestamp="select timestamp from event where cid=$EVENT_ID"
	mysql_exec_sensor_id="select sid from event where cid=$EVENT_ID"
	mysql_exec_event_signature="select signature from event where cid=$EVENT_ID"
	mysql_exec_signature_name="select sig_name from signature where sig_id=(select signature from event where cid=$EVENT_ID)"
	mysql_exec_sensor_hostname="select hostname from sensor where sid=(select sid from event where cid=$EVENT_ID);"
	mysql_exec_sensor_interface="select interface from sensor where sid=(select sid from event where cid=$EVENT_ID)"
	mysql_exec_ip_source="select inet_ntoa(ip_src) from iphdr where cid=$EVENT_ID"
	mysql_exec_ip_destination="select inet_ntoa(ip_dst) from iphdr where cid=$EVENT_ID"
	mysql_exec_tcp_source="select tcp_sport from tcphdr where cid=$EVENT_ID"
	mysql_exec_tcp_destination="select tcp_dport from tcphdr where cid=$EVENT_ID"
	mysql_exec_udp_source="select udp_sport from udphdr where cid=$EVENT_ID"
	mysql_exec_udp_destination="select udp_dport from udphdr where cid=$EVENT_ID"
	mysql_exec_icmp_type="select icmp_type from icmphdr where cid=$EVENT_ID"

	db_query

}


# Mit den vorbereiteten SQL-Statements werden die Informationen bzgl. eines Events aus der Snort-Datenbank gelesen
function db_query() {

	TIMESTAMP=`mysql --skip-column-names -h $SNORT_DB_HOST -P $SNORT_DB_PORT -u $SNORT_DB_USER --password=$SNORT_DB_PASS -e "$mysql_exec_timestamp" $SNORT_DB`
	SENSOR_ID=`mysql --skip-column-names -h $SNORT_DB_HOST -P $SNORT_DB_PORT -u $SNORT_DB_USER --password=$SNORT_DB_PASS -e "$mysql_exec_sensor_id" $SNORT_DB`
	EVENT_SIGNATURE=`mysql --skip-column-names -h $SNORT_DB_HOST -P $SNORT_DB_PORT -u $SNORT_DB_USER --password=$SNORT_DB_PASS -e "$mysql_exec_event_signature" $SNORT_DB`
	SIGNATURE_NAME=`mysql --skip-column-names -h $SNORT_DB_HOST -P $SNORT_DB_PORT -u $SNORT_DB_USER --password=$SNORT_DB_PASS -e "$mysql_exec_signature_name" $SNORT_DB`
	SENSOR_HOSTNAME=`mysql --skip-column-names -h $SNORT_DB_HOST -P $SNORT_DB_PORT -u $SNORT_DB_USER --password=$SNORT_DB_PASS -e "$mysql_exec_sensor_hostname" $SNORT_DB`
	SENSOR_INTERFACE=`mysql --skip-column-names -h $SNORT_DB_HOST -P $SNORT_DB_PORT -u $SNORT_DB_USER --password=$SNORT_DB_PASS -e "$mysql_exec_sensor_interface" $SNORT_DB`
	IP_SOURCE=`mysql --skip-column-names -h $SNORT_DB_HOST -P $SNORT_DB_PORT -u $SNORT_DB_USER --password=$SNORT_DB_PASS -e "$mysql_exec_ip_source" $SNORT_DB`
	IP_DESTINATION=`mysql --skip-column-names -h $SNORT_DB_HOST -P $SNORT_DB_PORT -u $SNORT_DB_USER --password=$SNORT_DB_PASS -e "$mysql_exec_ip_destination" $SNORT_DB`
	TCP_SOURCE=`mysql --skip-column-names -h $SNORT_DB_HOST -P $SNORT_DB_PORT -u $SNORT_DB_USER --password=$SNORT_DB_PASS -e "$mysql_exec_tcp_source" $SNORT_DB`
	TCP_DESTINATION=`mysql --skip-column-names -h $SNORT_DB_HOST -P $SNORT_DB_PORT -u $SNORT_DB_USER --password=$SNORT_DB_PASS -e "$mysql_exec_tcp_destination" $SNORT_DB`
	UDP_SOURCE=`mysql --skip-column-names -h $SNORT_DB_HOST -P $SNORT_DB_PORT -u $SNORT_DB_USER --password=$SNORT_DB_PASS -e "$mysql_exec_udp_source" $SNORT_DB`
	UDP_DESTINATION=`mysql --skip-column-names -h $SNORT_DB_HOST -P $SNORT_DB_PORT -u $SNORT_DB_USER --password=$SNORT_DB_PASS -e "$mysql_exec_udp_destination" $SNORT_DB`
	ICMP_TYPE=`mysql --skip-column-names -h $SNORT_DB_HOST -P $SNORT_DB_PORT -u $SNORT_DB_USER --password=$SNORT_DB_PASS -e "$mysql_exec_icmp_type" $SNORT_DB`

	build_alert
}

# Zusammensetzen der Alarmmeldung aus den Event-Informationen
function build_alert(){

	ALERT="

	{
	\"type\": \"Snort\",
	\"timestamp\": \"${TIMESTAMP}\",
	\"status\": \"OK\",
	\"ipsrc\": \"${IP_SOURCE}\",
	\"ipdest\": \"${IP_DESTINATION}\",
	\"protocol\": \"\",
	\"message\": \"${SIGNATURE_NAME}\",
	\"class\": \"${EVENT_SIGNATURE}\",
	\"data\":  
			{   \"sensor-id\": \"${SENSOR_ID}\",
		       	    \"event-id\": \"${EVENT_ID}\",
		            \"sensor-name\": \"${SENSOR_HOSTNAME}\",
		            \"interface\": \"${SENSOR_INTERFACE}\",
		            \"src\": \"${TCP_SOURCE}${UDP_SOURCE}\",
		            \"dest\": \"${TCP_DESTINATION}${UDP_DESTINATION}\",
		            \"icmp-type\": \"${ICMP_TYPE}\"
			}
		  
	}
	"

	start_nsca

}


# Übergibt die im Alarmmeldungen an den send_nsca.
function start_nsca(){

	# Aufruf des send_nsca
	/bin/echo -e "$HOSTNAME+$SERVICE_DESCRIPTION+$SERVICE_STATE+"$ALERT"\n" | send_nsca -b -H $ICINGA_HOST -d + -c /etc/send_nsca.cfg 1>/dev/null &

}

# Startet das Plugin.
get_last_snort_id


